
/**************************************************************************
 *                                                                        *
 * Copyright (c) 1996 Michael Richmond and Richard Treffers               *
 *                                                                        *
 *                    This software may be copied and distributed for     *
 *                    educational, research and not for profit services   *
 *                    provided that this copyright and statement are      *
 *                    included in all such copies.                        *
 *                                                                        *
 **************************************************************************/


/*	
	returns pointer to card image starting with instring
	symbol table name SYM_TABLE must be set in environment 
	strings of form
		xxx=yyy
	instring==xxx
	returns yyy or NULL if match is not found

	Author: R.R. Treffers

	Modified so that strings no longer translated into lower case when 
	put into the symbol table.  MWR 9/10/91.

	9/23/92 -remove get_float,put_float, and dead wood -rrt
	4/22/94 - cast strlen() 
	7/7/94 - add stdinclude for prototypes -rrt
*/

#include "pcvista.h"
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <stdlib.h>

#define LF		10
#define CR		13

char *
getsym(instring)		/* returns pointer to card image starting */
char *instring;				/* with instring */
{
	FILE *fp;
	int last;
	static char buf[NBUF], *rval;
	char *eq_loc,  *table;

	if ((table = getenv("SYM_TABLE")) == NULL)
		error(-1, "SYM_TABLE not set in environment");

	if ((fp = fopen(table, "r")) == NULL)
		return(NULL);

	rval = NULL;
	while (fgets(buf, NBUF, fp) != NULL) {
		eq_loc = strchr(buf, '=');			/* find location of equal sign */
		if (eq_loc != NULL){		
			*eq_loc = '\0';				/* convert buf to first string */
			if (!strcmp(buf, instring)) {
				rval = eq_loc + 1;			/* skip over = */
				break;
			}
		}	
	}
	if (rval != NULL) {
		last = strlen(rval) - 1;
		if (rval[last] == LF)	/* kill off dangling line feeds */
			rval[last] = 0;
	}
	fclose(fp);			
	return(rval);
}
	
/*	
	installs string in symbol table
	if second string is null, the symbol is removed
	calls error if failure
*/

void 
putsym(string)
char *string;
{
	FILE *fin, *fout;
	int nc;
	unsigned long pos;		/*** NEW ***/
	char buf[NBUF], *table;

	for (nc = 0; nc < (int)strlen(string); nc++)
		if (string[nc] == '=')
			break;

	if ((table = getenv("SYM_TABLE")) == NULL)
		error(-1, "SYM_TABLE not set in environment");

	/* we WILL check to see if 'fin' is NULL below... */
	fin = fopen(table, "r");

	if ((fout = fopen(table, "r+")) == NULL)
		if ((fout = fopen(table, "w")) == NULL)
			error(-1, "can't create symbol table");

	if (fin != NULL) {
		/* copy file skipping desired line */
		while (fgets(buf, NBUF, fin) != NULL) {
			if ((buf[nc] != '=') || strncmp(buf, string, nc)) {
				if (fputs(buf, fout) == EOF) {
					error(-1, "write #1 error in putsym");
				}
			}
		}
	}

	/* print only if a second value is present */
	if (nc < ((int)strlen(string) - 1))	
		fprintf(fout, "%s\n", string);
	fflush(fout);			
	pos = ftell(fout);			/* NEW */
#ifdef LINUX
	fclose(fout);
	truncate(table, pos);
#else
	ftruncate(fout->_file, pos);		/* truncate file */
	fclose(fout);
#endif
	fclose(fin);
}

