
/**************************************************************************
 *                                                                        *
 * Copyright (c) 1996 Michael Richmond and Richard Treffers               *
 *                                                                        *
 *                    This software may be copied and distributed for     *
 *                    educational, research and not for profit services   *
 *                    provided that this copyright and statement are      *
 *                    included in all such copies.                        *
 *                                                                        *
 **************************************************************************/


/*	
	window the file using a specific box -- that is, delete all parts of
	the image which are not inside the box

		WINDOW source BOX=n [OUTFILE=name] [help]

    if OUTFILE is specfied, the new, smaller image is placed into the
    given filename - otherwise, it overwrites the current image.

	9/22/92 - changed usage,exit(0), and used fits_copyheader()
	1/10/96 - added 'help' option.  MWR
*/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "pcvista.h"
#include "fits.h"

#ifdef PROTO
void main(int, char **);
#else
#endif

char *progname = "window";
char *usage = "usage: window fname box= [outfile=] [help]";

#define FITS_CARD_START	6		/* place to start copying FITS cards */

void
main(argc, argv)
int argc;
char *argv[];
{
	int nrow, ncol, nrbox, ncbox, nr, nc;
	int boxno = -1, i, sr, sc, got_outfile = 0;
	FITS_HANDLE fin, fout;
	char *gotit, buf[NBUF], foutname[NBUF];
	static int16 data[NMAX];

	if (argc < 3) {
		error(-1, usage);
	}
	if (strcmp(argv[1], "help") == 0) {
		printf("%s\n", usage);
		exit(0);
	}

	fin = fits_open(argv[1], "r", &nrow, &ncol);

	for (i = 2; i < argc; i++) {
		if (keyword("help", argv[i])) {
			printf("%s\n", usage);
			exit(0);
		}
		if ((gotit = find("box", argv[i])) != NULL) {
			boxno = (int) (evaluate(gotit) + 0.5);	
			continue;
		}
		if ((gotit = find("outfile", argv[i])) != NULL) {
			strcpy(foutname, gotit);
			got_outfile = 1;
			continue;
		}
		sprintf(buf, "unknown keyword '%s'", argv[i]);
		error(-1, buf);
	}
	
	if (getbox(boxno, &sr, &sc, &nrbox, &ncbox))
		error(-1, "box not found");
	nr = nrbox;
	nc = ncbox;
	check_box(boxno, nrow, ncol);

	if (got_outfile) {
		fout = fits_open(foutname, "w", &nr, &nc);
		fits_copyheader(fin, fout, FITS_NOCHECK);	
	}
	else		
		fout = fits_open(argv[1], "x", &nr, &nc);

	for (i = 0; i < nr; i++) {
			fits_get_data(fin, i + sr, sc, data, nc);
			fits_put_data_fast(fout, data, nc);
	}
	if (!got_outfile)
		fits_cut(fout);
	fits_close(fout);

	exit(0);
}

